/*
 * NOSA HEADER START
 *
 * The contents of this file are subject to the terms of the NASA Open 
 * Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may 
 * not use this file except in compliance with the Agreement.
 *
 * You can obtain a copy of the agreement at
 *   docs/NASA_Open_Source_Agreement_1.3.txt
 * or 
 *   http://cdaweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
 *
 * See the Agreement for the specific language governing permissions
 * and limitations under the Agreement.
 *
 * When distributing Covered Code, include this NOSA HEADER in each
 * file and include the Agreement file at 
 * docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the 
 * following below this NOSA HEADER, with the fields enclosed by 
 * brackets "[]" replaced with your own identifying information: 
 * Portions Copyright [yyyy] [name of copyright owner]
 *
 * NOSA HEADER END
 *
 * Copyright (c) 2011 United States Government as represented by 
 * the National Aeronautics and Space Administration. No copyright is 
 * claimed in the United States under Title 17, U.S.Code. All Other 
 * Rights Reserved.
 *
 * $Id: JDnDList.java,v 1.5 2012/03/08 15:58:57 bharris Exp $
 */
package gsfc.spdf.edit.guis;


import javax.swing.JList;
import java.awt.dnd.*;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.DataFlavor;
import java.util.StringTokenizer;
import java.util.*;
import java.io.IOException;
import java.awt.datatransfer.UnsupportedFlavorException;
import gsfc.nssdc.cdf.*;
import gsfc.nssdc.cdf.util.*;
import javax.swing.SwingUtilities;
import java.awt.Cursor;
/**
 *
 * @author rchimiak
 */
public class JDnDList extends JList implements DragSourceListener, 
            DragGestureListener, DropTargetListener
{ 
    
    DragSource dragSource = null;
    DropTarget dropTarget = null;

    private int[] selectedIndices = null;
    private boolean dragging;
    private int overIndex;
    private SKTEditor myEditor;


    
    public JDnDList( DnDListModel myModel, SKTEditor myEditor )
    {
        super();
        this.setModel(myModel);
        this.myEditor = myEditor;
        
 
        // Create a drag source
        dragSource = new DragSource();
        
        //Ask the drag source to watch a specific component for drag gestures
        dragSource.createDefaultDragGestureRecognizer( this, DnDConstants.ACTION_MOVE, this);
        
        // Create a drop target
        dropTarget = new DropTarget( this, this );
    }

    //call when a drag gesture is recognized
    public void dragGestureRecognized(DragGestureEvent dge) {
        
        this.selectedIndices = this.getSelectedIndices();
 	Object[] selectedObjects = this.getSelectedValues();
        if( selectedObjects.length > 0 ){
   
            StringBuffer sb = new StringBuffer();
            for( int i=0; i<selectedObjects.length; i++ ) {
                
                sb.append( selectedObjects[ i ].toString() + "\n" );
            }

            // StringSelection implements Transferable.
            //Build a StringSelection object that the Drag Source
            // can use to transport a string to the Drop Target
            StringSelection text = new StringSelection( sb.toString() ); 

            // Ask the drag source to start the drag of our text 
            this.dragging = true;
            dragSource.startDrag( dge, DragSource.DefaultMoveDrop, text, this );
        }
    }
    
    //Listen to all notifications when our object is dragged
    //over a drio target
    public void dragEnter(DragSourceDragEvent dsde)
    {
        
    }
    

    
    public void dragOver(DragSourceDragEvent dsde)
    {
        
    }
                
    public void dropActionChanged(DragSourceDragEvent dsde)
    {
        
    }
    public void dragExit(DragSourceEvent dsde)
    {
        
    }
    
 
    public void dragExit(DropTargetEvent dte) {
        
        this.overIndex = -1;
    }
 
    public void dragEnter(DropTargetDragEvent dtde) {
    
        this.overIndex = this.locationToIndex( dtde.getLocation() );
        this.setSelectedIndex( this.overIndex );
  }
 
    public void dragOver(DropTargetDragEvent dtde) {
    
        // See who we are over...
        int overIndex = this.locationToIndex( dtde.getLocation() );
        if( overIndex != -1 && overIndex != this.overIndex ){
            
            // If the value has changed from what we were previously over
            // then change the selected object to the one we are over; this 
            // is a visual representation that this is where the drop will occur
            this.overIndex = overIndex;
            this.setSelectedIndex( this.overIndex );
        }
    }
    
    public void dropActionChanged(DropTargetDragEvent dtde){;}

    //handle the drop
     public void dragDropEnd(DragSourceDropEvent dsde) {
    
         this.dragging = false;
    }
    
     public void drop(DropTargetDropEvent dtde) {
    
         try 
        {
             Transferable transferable = dtde.getTransferable();
             
              int newIndex;
            if( transferable.isDataFlavorSupported( DataFlavor.stringFlavor ) )
            {                
                dtde.acceptDrop( DnDConstants.ACTION_MOVE );

                // Find out where the item was dropped
                newIndex = this.locationToIndex( dtde.getLocation() );

                // Get the items out of the transferable object and build an
                // array out of them...
                String s = ( String )transferable.getTransferData( DataFlavor.stringFlavor );
                StringTokenizer st = new StringTokenizer( s );
                ArrayList items = new ArrayList();
                while( st.hasMoreTokens() )
                {
                    items.add( st.nextToken() );
                }
                
                
                DnDListModel model = ( DnDListModel )this.getModel();
                 int oldIndex = this.selectedIndices[0];
                // If we are dragging from our this to our list them move the items,
                // otherwise just add them...
                if( this.dragging )
                {
                    //model.itemsMoved( newIndex, items );
                    model.itemsMoved( newIndex, this.selectedIndices );
                }
 
                // Update the selected indicies
                int[] newIndices = new int[ items.size() ];
                for( int i=0; i<items.size(); i++ )
                {
                    newIndices[ i ] = newIndex + i;
                }
                this.setSelectedIndices( newIndices );

                // Reset the over index
                this.overIndex = -1;

                dtde.getDropTargetContext().dropComplete( true );
                
                try{
                
               CDF cdf =  myEditor.theCDF;
               
               	if (myEditor.theCDF != null){
                   
                   Vector v = myEditor.theCDF.getVariables();
                   
//                   Vector clone = (Vector)v.clone();
                   
                   Variable var = (Variable)v.get(oldIndex);

                   String name = var.getName();
                   var.duplicate(var.getName() + "tmp");
                   var.delete();
                   ((Variable)v.get(v.size()-1)).rename(name);
                   
                   int j = v.size()-1 - newIndex;
                   while (j >0){
                       j--;
                       
                      var = (Variable)v.get(newIndex); 

                      name = var.getName();
                      var.duplicate(var.getName() + "tmp");
                      var.delete();
                      ((Variable)v.get(v.size()-1)).rename(name);
                   }                                       
                   
                  }
                }catch(CDFException cdfex){;}
	    
          
            }
            else
            {
                dtde.rejectDrop();
            }
        }
        catch( IOException exception )
        {
            exception.printStackTrace();
            System.err.println( "Exception" + exception.getMessage());
            dtde.rejectDrop();
        }
        catch( UnsupportedFlavorException ufException )
        {
            ufException.printStackTrace();
            System.err.println( "Exception" + ufException.getMessage());
            dtde.rejectDrop();
        }
    }

}


 

 
