/*
 * NOSA HEADER START
 *
 * The contents of this file are subject to the terms of the NASA Open 
 * Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may 
 * not use this file except in compliance with the Agreement.
 *
 * You can obtain a copy of the agreement at
 *   docs/NASA_Open_Source_Agreement_1.3.txt
 * or 
 *   http://cdaweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
 *
 * See the Agreement for the specific language governing permissions
 * and limitations under the Agreement.
 *
 * When distributing Covered Code, include this NOSA HEADER in each
 * file and include the Agreement file at 
 * docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the 
 * following below this NOSA HEADER, with the fields enclosed by 
 * brackets "[]" replaced with your own identifying information: 
 * Portions Copyright [yyyy] [name of copyright owner]
 *
 * NOSA HEADER END
 *
 * Copyright (c) 2011 United States Government as represented by 
 * the National Aeronautics and Space Administration. No copyright is 
 * claimed in the United States under Title 17, U.S.Code. All Other 
 * Rights Reserved.
 *
 * $Id: VerificationResult.java,v 1.4 2011/03/18 10:28:06 bharris Exp $
 */
package gsfc.spdf.istp;

import java.util.Vector;
//import java.util.Hashtable;
import java.util.Map;
/**
 * This class represents the results of verifying a CDF's ISTP 
 * compliance.
 *
 * @author B. Harris
 * @version $Revision: 1.4 $
 */
public class VerificationResult {

    /**
     * Global attribute error messages.
     */
    private Vector globalAttributeErrors = null;

    /**
     * Variable error messages.
     */
 //   private Hashtable variableErrors = null;
private Map variableErrors = null;
    /**
     * Status of ISTP compliance verification operation.
     */
    private int status = COMPLETE;

    /**
     * ISTP compliance verification status indicating that the operation
     * was completed (with or without errors).
     */
    public static final int COMPLETE = 0;

    /**
     * ISTP compliance verification statis indicating that the operation
     * was aborted.  Presently, the verification operation will always
     * complete unless instructed to abort by the
     * {@link VerificationCallback}.
     */
    public static final int ABORTED = 1;


    /**
     * Creates a VerificationResult with the given values.
     *
     * @param globalAttributeErrors global attribute error messages
     * @param variableErrors variable error messages
     * @param status verification completion status
     */
    public VerificationResult(Vector globalAttributeErrors, 
                            //  Hashtable variableErrors, int status) {        
                                Map variableErrors, int status) {
        
        this.globalAttributeErrors = globalAttributeErrors;
        this.variableErrors = variableErrors;
        this.status = status;
    }


    /**
     * Gets the global attribute error messages.
     *
     * @return a vector containing the error messages concerning global
     *     attributes
     * @see #setGlobalAttributeErrors
     */
    public Vector getGlobalAttributeErrors() {

        return (Vector)globalAttributeErrors.clone();
    }


    /**
     * Sets the global attribute error messages.
     *
     * @param value a vector containing the error messages concerning 
     *            global attributes
     * @see #getGlobalAttributeErrors
     */
    public void setGlobalAttributeErrors(Vector value) {

        globalAttributeErrors = value;
    }


    /**
     * Gets the variable attribute error messages.
     *
     * @return a hashtable containing the error messages concerning 
     *     variables attributes.  The table's key value is the 
     *     variable's name and the associated value is a Vector 
     *     containing the actual error messages.
     * @see #setVariableErrors
     */
    public Map getVariableErrors() {

        return variableErrors;
    }


    /**
     * Sets the variable attribute error messages.
     *
     * @param value a hashtable containing the error messages 
     *            concerning variables.  The table's key value is the 
     *            variable's name and the associated value is a Vector 
     *            containing the actual error messages.
     * @see #getVariableErrors
     */
    public void setVariableErrors(Map value) {

        variableErrors = value;
    }


    /**
     * Gets the verification completion status.
     *
     * @return the verification completion status
     * @see #setStatus
     */
    public int getStatus() {

        return status;
    }


    /**
     * Sets the verification completion status.
     *
     * @param value the new verification completion status
     * @see #getStatus
     */
    public void setStatus(int value) {

        status = value;
    }
}

